/*
 * DEBUG.C - debugging aids.
 *
 * $Header:   C:/FE/SHARED/VCS/DEBUG.C_V   1.0   07 Dec 1988 23:09:36   rji  $
 *
 * $Log:   C:/FE/SHARED/VCS/DEBUG.C_V  $
 * 
 *    Rev 1.0   07 Dec 1988 23:09:36   rji
 * Initial revision.
 * 
*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
   $NAME$:
         DEBUG.C Includes

  $PATHS$
       Functions\C
       Shared\Includes
       Shared\Utils
       Modules\$./FL$
   $3$

   See Also:
        $/SEE(DEBUG.H Macros)$
        $/SEE(COMMON.H Macros)$

   $1$

  $0$
  SYNOPSIS:
  $-1$
*/
/*-------------------------------------------------------------------------*/
/* INCLUDE FILES                                                           */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>

#include <common.h>
#include <debug.h>
/*                                                                   $end$ */


/*-------------------------------------------------------------------------*/
/* PUBLIC DATA                                                             */
/*-------------------------------------------------------------------------*/

#if !defined(NDEBUG)
int   DBUG = 0;
#endif


/*-------------------------------------------------------------------------*/
/* PUBLIC FUNCTIONS                                                        */
/*-------------------------------------------------------------------------*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
   $NAME$:
         _hexdump

  $PATHS$
       Functions\C
       Shared\Procedures
       Shared\Utils
       Modules\$./FL$


   $1$

  $0$
  SYNOPSIS:
  $-1$
*/
#if !defined(NDEBUG)
/*-------------------------------------------------------------------------*/
/* _HEXDUMP                                                                */
/*                                                                         */
/* Write hexadecimal dump of buffer to file.                               */
/*-------------------------------------------------------------------------*/

#define  DEFAULT_NCHARS_PER_LINE   75  /* default # of characters per line */
#define  MAX_NCHARS_PER_LINE      255  /* maximum # of characters per line */

void  _hexdump( fp, p, nbytes, linesz )
   FILE    *fp;
   char    *p;
   size_t   nbytes;
   size_t   linesz;
/*                                                                   $end$ */
{
   REG2 int    start;               /* offset of start byte in dump buffer */
   REG1 int    j;                   /* loop control variable               */
   int         i;                   /* loop control variable               */
   int         nlines;              /* # blocks needed for hex dump        */

   static char byte_rule[] =        /* N.B. only up to 255 characters/line */
      "....5...10...15...20...25...30...35...40...45...50...55...60...65"
      "...70...75...80...85...90...95..100..105..110..115..120..125..130"
      "..135..140..145..150..155..160..165..170..175..180..185..190..195"
      "..200..205..210..215..220..225..230..235..240..245..250..255";

   /* use default output file and line size if not explicitly specified    */

   if ( fp == NULL )
      fp = stderr;

   if ( linesz == 0 )
      linesz = DEFAULT_NCHARS_PER_LINE;
   else if ( linesz > MAX_NCHARS_PER_LINE )
      linesz = MAX_NCHARS_PER_LINE;

   nlines = (nbytes + linesz - 1) / linesz;     /* calc # hex dump "lines" */

   for ( i = 0; i < nlines; ++i )
   {
      /* calc offset of 1st byte in current line & current line size */

      start = i * linesz;
      linesz = min( linesz, (nbytes - start) );

      /* for each line of input, o/p input line, hex codes, and byte-rule  */

      for ( j = 0; j < linesz; ++j )
         fprintf( fp, "%c", isprint( p[start+j] ) ? p[start+j] : '.' );
      fprintf( fp, "\n" );
      for ( j = 0; j < linesz; ++j )
         fprintf( fp, "%1X", ((unsigned char) p[start+j]) >> 4 );
      fprintf( fp, "\n" );
      for ( j = 0; j < linesz; ++j )
         fprintf( fp, "%1X", p[start+j] & 0x0f );
      fprintf( fp, "\n" );
      fprintf( fp, "%*.*s\n\n", linesz, linesz, byte_rule );
   }

} /* _hexdump() */
#endif



/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
   $NAME$:
         DEBUG.C main() Test Driver

  $PATHS$
       Functions\C
       Shared\Procedures
       Shared\Utils
       Modules\$./FL$


   $1$

  $0$
  SYNOPSIS:
  $-1$
*/
#if defined(DRIVER)
/*=========================================================================*/
/*=                                                                       =*/
/*=                       T E S T   D R I V E R                           =*/
/*=                                                                       =*/
/*=========================================================================*/

void main(int, char *[]);

void  main( argc, argv  )
   int   argc;
   char  *argv[];
/*                                                                   $end$ */
{
   char *progname = *argv;
   FILE *out      = NULL;           /* user-requested hex dump output file */
   int   linesz   = 0;              /* user-requested maximum line length  */
   int   i        = 0;              /* index to binput file buffer         */
   int   c;                         /* current input character             */

   static char buf[2000];           /* input file buffer                   */

   /* process command line arguments */

   while ( --argc > 0  &&  (*++argv)[0] == '-' )
      switch ( (*argv)[1] )
      {
         case 'n':
            linesz = atoi( *argv+2 );
            break;
         case 'o':
            if ( (out = fopen( *argv+2, "w" )) == NULL )
            {
               printf( "%s: cannot open \"%s\"\n",
                        progname,         *argv+2 );
               exit( 1 );
            }
            break;
         case 'D':
            set_debug( 1 );
            break;
         case '?':
         default:
            fprintf( stderr, "Usage: %s [-n<line_size>] [-o<output_file>] "
                             "[<file_name>]\n", progname );
            exit( 1 );
      }

   /* set up input file to dump */

   if ( argc > 0 )
      if ( freopen( *argv, "rb", stdin ) == NULL )
      {
         printf( "%s: cannot open \"%s\" not opened\n", progname, *argv );
         exit( 1 );
      }

   /* read in file (only up to buffer capacity) and hex dump it */

   while ( i < sizeof( buf )  &&  (c = fgetc( stdin )) != EOF )
      buf[i++] = (char) c;

   _hexdump( out, buf, i, linesz );

} /* main() */
#endif /* #if defined(DRIVER) */
